import os
from ftplib import FTP
import traceback

class MyFtp:

    ftp = FTP()

    def __init__(self,host,port=21,debugLevel=2):
        self.ftp.connect(host,port)
        self.ftp.set_debuglevel(debugLevel)  # 打开调试级别2，显示详细信息

    def login(self,username,pwd):
        
        self.ftp.login(username,pwd)
#         print(self.ftp.welcome)

    def downloadFile(self,localpath,remotepath,filename,isOverwrite=True):
        dstPath = os.path.join(localpath, filename)
        if os.path.exists(dstPath):
            if not isOverwrite:
                print(dstPath, "exists")

            else:
                # os.chdir(localpath)   # 切换工作路径到下载目录
                self.ftp.cwd(remotepath)  # 要登录的ftp目录
                self.ftp.nlst()  # 获取目录下的文件
                file_handle = open(dstPath,"wb").write   # 以写模式在本地打开文件
                self.ftp.retrbinary('RETR %s' % os.path.basename(filename),file_handle,blocksize=1024)  # 下载ftp文件
                # ftp.delete（filename）  # 删除ftp服务器上的文件


        else:
            # os.chdir(localpath)   # 切换工作路径到下载目录
            self.ftp.cwd(remotepath)  # 要登录的ftp目录
            self.ftp.nlst()  # 获取目录下的文件
            file_handle = open(dstPath, "wb").write  # 以写模式在本地打开文件
            self.ftp.retrbinary('RETR %s' % os.path.basename(filename), file_handle, blocksize=1024)  # 下载ftp文件
            # ftp.delete（filename）  # 删除ftp服务器上的文件 

    def upload_file(self,src_file, des_file):
        try:
            des_dirs = self.get_dirs(des_file)
            print(des_dirs)
            for d in des_dirs:
                if d not in self.ftp.nlst():
                    self.ftp.mkd(d)
                self.ftp.cwd(d)
            bufsize = 1024
            f = open(src_file, 'rb')
            des_file_name = os.path.basename(des_file)
            self.ftp.storbinary('STOR '+des_file_name, f, bufsize)
            print('upload %s success! '% des_file)
        except Exception as e:
            print('upload %s failed! ' % des_file)
            print(traceback.format_exc())
    def upload_fileTQW(self,src_file, des_path,fileName,reletive_path=""):
        try:
            self.ftp.cwd(des_path)
            for d in reletive_path.split(os.path.sep):
                if d not in self.ftp.nlst():
                    self.ftp.mkd(d)
                self.ftp.cwd(d)            
            bufsize = 1024
            f = open(src_file, 'rb')
            des_file_name = os.path.basename(fileName)
            self.ftp.storbinary('STOR '+des_file_name, f, bufsize)
            print('upload %s success! '% des_file_name)
        except Exception as e:
            print('upload %s failed! ' % des_file_name)
            print(traceback.format_exc())

    def get_dirs(self,filepath, isFile=True):

        dirs = filepath.split(os.path.sep)
        if isFile:
            return dirs[:-1]
        else:
            return dirs

    def close(self):
        self.ftp.set_debuglevel(0)  # 关闭调试
        self.ftp.quit()




import datetime
import numpy as np
import pandas as pd
from hjn.mkNCHJN import envelope
from sklearn.neighbors import KDTree
import matplotlib.pyplot as plt

class IDW():
    def __init__(self, ltc, y, x, z, resolution=0.01, candidate=15):
        self.evn = ltc
        self.resolution = resolution
        self.latRange = int((self.evn.n - self.evn.s + 10e-10) // self.resolution) + 1
        self.lonRange = int((self.evn.e - self.evn.w + 10e-10) // self.resolution) + 1
        self.latArr = np.linspace(self.evn.n, self.evn.s, self.latRange)
        self.lonArr = np.linspace(self.evn.w, self.evn.e, self.lonRange)
        self.disMat = None
        self.idMat = None
        self.mat = np.full([self.latRange, self.lonRange], 0, dtype=np.float)
        self.x = x
        self.y = y
        self.z = z
        self.resultMat = None
        self.candidate = candidate

    def initDict(self):
        latArr = np.linspace(self.evn.n, self.evn.s, self.latRange)
        lonArr = np.linspace(self.evn.w, self.evn.e, self.lonRange)
        lonMat, latMat = np.meshgrid(lonArr, latArr)
        tree = KDTree(np.c_[self.y, self.x])
        xyMesh = np.c_[latMat.reshape(-1), lonMat.reshape(-1)]
        dist, ind = tree.query(xyMesh, k=self.candidate)
        self.disMat = dist.reshape([len(latArr), len(lonArr), self.candidate])
        self.idMat = ind.reshape([len(latArr), len(lonArr), self.candidate])

    def getMat(self):
        valMat = self.z.values[self.idMat]
        disAll = np.sum(1 / np.power(self.disMat, 2), axis=2)
        dataAll = np.sum(valMat / np.power(self.disMat, 2), axis=2)
        self.resultMat = dataAll / disAll
        self.resultMat[self.disMat[:, :, 0] == 0] = valMat[self.disMat[:, :, 0] == 0, 0]
        return self.resultMat





# -*- coding: utf-8 -*-

from collections import namedtuple
from operator import itemgetter
from pprint import pformat
import numpy as np


class Node(namedtuple('Node', 'location left_child right_child')):
    def __repr__(self):
        return pformat(tuple(self))


class KDTree():
    def __init__(self, points):
        self.tree = self._make_kdtree(points)
        if len(points) > 0:
            self.k = len(points[0])
        else:
            self.k = None

    def _make_kdtree(self, points, depth=0):
        if not points:
            return None

        k = len(points[0])
        axis = depth % k

        points.sort(key=itemgetter(axis))
        median = len(points) // 2

        return Node(
            location=points[median],
            left_child=self._make_kdtree(points[:median], depth + 1),
            right_child=self._make_kdtree(points[median + 1:], depth + 1))

    def find_nearest(self,
                     point,
                     root=None,
                     axis=0,
                     dist_func=lambda x, y: np.linalg.norm(x - y)):

        if root is None:
            root = self.tree
            self._best = None

        # 若不是叶节点，则继续向下走
        if root.left_child or root.right_child:
            new_axis = (axis + 1) % self.k
            if point[axis] < root.location[axis] and root.left_child:
                self.find_nearest(point, root.left_child, new_axis)
            elif root.right_child:
                self.find_nearest(point, root.right_child, new_axis)

        # 回溯：尝试更新 best
        dist = dist_func(root.location, point)
        if self._best is None or dist < self._best[0]:
            self._best = (dist, root.location)

        # 若超球与另一边超矩形相交
        if abs(point[axis] - root.location[axis]) < self._best[0]:
            new_axis = (axis + 1) % self.k
            if root.left_child and point[axis] >= root.location[axis]:
                self.find_nearest(point, root.left_child, new_axis)
            elif root.right_child and point[axis] < root.location[axis]:
                self.find_nearest(point, root.right_child, new_axis)

        return self._best

if __name__ == '__main__':
    point_list = [(2, 3), (5, 4), (9, 6), (4, 7), (8, 1), (7, 2)]
    kdtree = KDTree(point_list)

    point = np.array([5, 5])
    print(kdtree.find_nearest(point))



from PIL import Image, ImageDraw
import matplotlib.pyplot as plt
from hjn.mkNCHJN import envelope
import numpy as np
import netCDF4 as nc
import copy
import geopandas as gpd

class maskClip():
    def __init__(self,data,latAtt,lonArr,step):

        self.dem = data
        self.latArr = latAtt
        self.lonArr = lonArr

        oriStep=np.abs((self.latArr[0]-self.latArr[-1])/(len(self.latArr)-1))
        range=int(np.round(step/oriStep,0))
        self.dem=self.dem[::range,::range]

        self.ltc = envelope(self.latArr[0], self.latArr[-1], self.lonArr[0], self.lonArr[-1])
        self.step = step

    def world2Pixel(self,ltc, x, y):
        """
        Uses a gdal geomatrix (gdal.GetGeoTransform()) to calculate
        the pixel location of a geospatial coordinate
        """
        ulX = ltc.w
        ulY = ltc.n
        xDist = self.step

        pixel = int((x - ulX) / xDist)
        line = int((ulY - y) / xDist)
        return (pixel, line)


    def getMask(self,geom,minX,maxY,minY,maxX):

        points = []
        pixels = []
        ulX, ulY = self.world2Pixel(self.ltc, minX, maxY)
        lrX, lrY = self.world2Pixel(self.ltc, maxX, minY)

        # Calculate the pixel size of the new image
        pxWidth = int(lrX - ulX)
        pxHeight = int(lrY - ulY)

        ltc1 = copy.copy(self.ltc)
        ltc1.n = maxY
        ltc1.w = minX

        pts = geom.boundary.xy
        for p in range(len(pts[0])):
            points.append((pts[0][p], pts[1][p]))
        for p in points:
            pixels.append(self.world2Pixel(ltc1, p[0], p[1]))
        rasterPoly = Image.new("L", (pxWidth, pxHeight), 1)
        rasterize = ImageDraw.Draw(rasterPoly)
        if len(pixels) > 1:
            rasterize.polygon(pixels, 0)

        mask = np.asarray(rasterPoly)


        # plt.imshow(mask)
        # plt.show()

        latOffset0 = int((self.ltc.n - maxY) / self.step)
        latOffset1 = self.dem.shape[0] - int((self.ltc.n - minY) / self.step)
        lonOffset0 = int((minX - self.ltc.w) / self.step)
        lonOffset1 = self.dem.shape[1] - int((maxX - self.ltc.w) / self.step)
        ndarray = np.pad(mask, ((latOffset0, latOffset1),
                                (lonOffset0, lonOffset1)), 'constant', constant_values=(1, 1))

        clip = copy.copy(self.dem)

        clip[ndarray != 0] = np.nan

        return clip



    def clip(self,shapefile_path,encoding='gb18030'):
        chly = gpd.GeoDataFrame.from_file(shapefile_path, encoding=encoding).geometry

        minX, maxX, minY, maxY = chly.bounds.min().minx,chly.bounds.max().maxx,chly.bounds.min().miny,chly.bounds.max().maxy
        maskArr=[]

        for j in range(len(chly)):
            geom = chly[j]
            if geom.geometryType() == 'Polygon':
                clip=self.getMask(geom, minX, maxY, minY, maxX)
                maskArr.append(clip)
            elif geom.geometryType() == "MultiPolygon":
                multiLayer = []
                for i in range(len(geom)):
                    geom1 = geom[i]
                    clip = self.getMask(geom1, minX, maxY, minY, maxX)
                    multiLayer.append(clip)

                multiLayer= np.asarray(multiLayer)
                clip = np.nanmax(multiLayer,axis=0)
                maskArr.append(clip)

        return maskArr


class slope_aspect():
    def __init__(self,gridNew,dx, dy):
        self.gridNew = gridNew
        self.dx = dx
        self.dy = dy
        self.Sx, self.Sy = self.calcFiniteSlopes(gridNew, self.dx, self.dy)
    # 计算dx,dy
    def calcFiniteSlopes(self,elevGrid, dx, dy):
        Zbc = np.pad(elevGrid, ((1, 1), (1, 1)), "edge")
        Sx = (Zbc[1:-1, :-2] - Zbc[1:-1, 2:]) / (2 * dx)  # WE方向
        Sy = (Zbc[2:, 1:-1] - Zbc[:-2, 1:-1]) / (2 * dy)  # NS方向
        return Sx, Sy

    def calcFiniteAspect(self,Sx, Sy):

        Sx = Sx.astype(np.float)
        Sy = Sy.astype(np.float)
        aspect = np.degrees(np.arctan2(Sy, Sx))
        aspect1 = copy.copy(aspect)
        aspect1[aspect < 0] = 90 - aspect[aspect < 0]
        aspect1[aspect > 90] = 360.0 - aspect[aspect > 90] + 90.0
        aspect1[np.logical_and(aspect <= 90, aspect >= 0)] = 90.0 - aspect[np.logical_and(aspect <= 90, aspect >= 0)]
        aspect1[np.logical_and(Sx == 0, Sy == 0)] = -1
        return aspect1

    def getSlop(self):
        return np.degrees(np.arctan(np.sqrt(self.Sx ** 2 + self.Sy ** 2)))

    def getAspect(self):
        return self.calcFiniteAspect(self.Sx, self.Sy)




mport hashlib
import time
def DAA_example():
    t = time.time()
    para = "serviceNodeId=NMIC_MUSIC_CMADAAS&userId=&interfaceId=getSurfEleByTime&dataCode=SURF_CHN_WSET_FTM&elements=Cnty&times=20200826000000&dataFormat=csv"
    timeStep = "timestamp=%s000"%int(t)
    noncemd5 = hashlib.md5()
    noncemd5.update(timeStep.encode('utf-8'))
    nonce="nonce=%s"%(noncemd5.hexdigest().upper())
    pwd = "pwd="
    paraSK = "&".join([timeStep,nonce,pwd])
    allPara = para+"&"+paraSK
    list= allPara.split("&")
    list.sort()
    paramSort = "&".join(list)
    md5 = hashlib.md5()
    md5.update(paramSort.encode('utf-8'))
    sign ="sign="+md5.hexdigest().upper()
    prefix = "http://10.40.17.54/music-ws/api?"
    url = prefix + para + "&"+"&".join([timeStep,nonce,sign])
    print(url)

if __name__ == '__main__':
    DAA_example()


